/* ============================================================================
 * File Name: Button_sw.c
 *   Development version of the Button_SW component
 *
 * Description:
 *   Demo project of button swich detection using  
 *   vertical counters algorithm
 *   Uses polling technique.
 *
 * Credits:
 *   based on vertical counters algorithm
 *    https://www.compuphase.com/electronics/debouncing.htm
 *
 * Note:
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/



   
#include <button_sw.h> // must specify API prefix in Symbol->Properties->Doc.APIPrefix



#define BTN1_MASK    (~(0xFF << BTN1_NumButtons))   // botton mask 


//====================================
//        private variables
//====================================
volatile uint8 fBtnPressed_latched;                   // flag indicating button pressed
volatile uint8 fBtnReleased_latched;                  // flag indicating button released



//====================================
//        private functions
//====================================
//CY_ISR_PROTO(BTN1_myPollInt);   // interrupt on external polling clock
void  BTN1_CheckStatus();       // return: 

 
//==============================================================================
// STATE CHECK USING POLLING METHOD->
// interrupt on external clock for polling states
//==============================================================================

CY_ISR(BTN1_myPollInt) 
{   
    BTN1_CheckStatus(); 
} 
 

//==============================================================================
// Button state is continuously polled. 
// If state is repeated 3 times it considered as debounced
//==============================================================================

__INLINE void BTN1_CheckStatus() // todo: need return value?
{
    static uint8 fBtnPressed;                           // flag indicating button pressed
    static uint8 fBtnReleased;                          // flag indicating button released
    
    static unsigned char state, cnt0, cnt1;
    unsigned char delta, toggle; // (v1)
    //unsigned char delta; // (v3)
    

    // here the logic is inverted: when button is pressed, voltage goes to 0V,
    // but vertical counter expects logical HIGH, so we invert the signal
    
    uint8 sample = ~(Pin_BTN1_Read()) & BTN1_MASK;  // read pins state (port), pressed = 1, 
    
    
    
    // 3. refined method->
    // this code returns both the state 0,1 and transition 01, 10
    
    delta = sample ^ state;
    cnt1 = (cnt1 ^ cnt0) & delta;
    cnt0 = ~cnt0 & delta;

    toggle = delta & ~(cnt0 | cnt1);
    state ^= toggle;
    
    //fBtnPressed = state;
    //fBtnPressed = toggle; // on state change 
    fBtnPressed = toggle & state; // on rising edge
    //fBtnPressed = toggle & ~state; // on falling  edge
    fBtnReleased = toggle & ~state; // on falling edge
    
    fBtnPressed_latched  |= fBtnPressed;  // latch pressed
    fBtnReleased_latched |= fBtnReleased; // latch release
}



//==============================================================================
// Read and reset button pressed flag
// return: 1- pressed, 0-none
//==============================================================================

uint8 BTN1_GetBtnPressed()  
{
    isrPoll_Disable(); // to prevent sudden fBtnPressed_latched change in isr
    
    uint8 result = fBtnPressed_latched; // get result
    fBtnPressed_latched = 0;            // clear latch
    
    isrPoll_Enable();
    
    return result;
}

//==============================================================================
// Read and reset button released flag 
// return: 1- releaased, 0-none
//==============================================================================

uint8 BTN1_GetBtnReleased()  
{
    isrPoll_Disable(); // to prevent sudden fBtnRelease_latched change in isr
    
    uint8 result = fBtnReleased_latched; // get result
    fBtnReleased_latched = 0;            // clear latch
    
    isrPoll_Enable();
    
    return result;
}



//==============================================================================
// Initialize encoder 
//==============================================================================

void BTN1_Start()
{ 
    isrPoll_StartEx(BTN1_myPollInt); // start isrPoll interrupt
} 

//==============================================================================
// Stop encoder 
//==============================================================================

void BTN1_Stop()
{
    isrPoll_Stop();    // staop isrPoll interrupt
} 

//==============================================================================
//==============================================================================
/* [] END OF FILE */









/*

 There is explanation of the code from website:
 https://www.compuphase.com/electronics/debouncing.htm

unsigned char debounce(unsigned char sample)
{
    static unsigned char state, cnt0, cnt1;
    unsigned char delta, toggle;

    delta = sample ^ state;

    cnt1 = cnt1 ^ cnt0;
    cnt0 = ~cnt0;

    cnt0 &= delta;
    cnt1 &= delta;

    toggle = cnt0 & cnt1;
    state ^= toggle;

    return state;
}

Line 1
    The argument sample holds the raw state of the input pins (the un-debounced state).

Line 3
    The routine requires three fields with persistent state: two fields to form a 2-bit vertical counter
    and one field to hold the current (debounced) state of the inputs.

Line 6
    The variable delta contains a 1 bit for every input that is different from the current debounced state.

Lines 8 & 9
    These lines update the vertical counter. Line 9 toggles all bits of counter cnt0 (our “vertical bit” 0)
    and line 8 toggles the bits in cnt1 depending on the old value of cnt0. Since cnt1 depends on the
    previous value of cnt0, cnt1 is updated before cnt0.

Lines 11 & 12
    Any bits in the sample that are the same as that of the current debounced state are cleared. One 
    consequence is that the vertical counter (in cnt0 and cnt1) is only incremented for those bits where
    the sample is different from the debounced state. A second consequence is that when there is a “contact
    bounce” in the samples where a sample bit momentarily toggles back to the old state, the vertical counter
    for that bit is reset to zero.
    Note that the operations on lines 11 and 12 can be combined with those on lines 8 and 9, for 
    more compact code.

Line 14
    Variable toggle gets a 1 bit at every position in which cnt0 and cnt1 have a one bit —that is, when
    the vertical count (for that bit position) is 3. Since the vertical count is reset on a contact bounce,
    a vertical count of 3 means that the respective bit has remained stable for 3 consecutive samples. So,
    variable toggle has a 1 bit for every bit that must be toggled in the output state.

Line 15
    Those bits that have been detected as “changed”, are toggled in the variable for the debounced state.
    This will become the new debounced state. This operation can be combined with line 14, which also
    saves a local variable.

Line 17
    The function returns the debounced state.


//same but Compact implementation
unsigned char debounce(unsigned char sample)
{
    static unsigned char state, cnt0, cnt1;
    unsigned char delta;

    delta = sample ^ state;
    cnt1 = (cnt1 ^ cnt0) & delta;
    cnt0 = ~cnt0 & delta;
    state ^= (cnt0 & cnt1);

    return state;
}

// A refined debouncing routine
unsigned char debounce(unsigned char sample, unsigned char *toggle)
{
    static unsigned char state, cnt0, cnt1;
    unsigned char delta;

    delta = sample ^ state;
    cnt1 = (cnt1 ^ cnt0) & delta;
    cnt0 = ~cnt0 & delta;

    *toggle = delta & ~(cnt0 | cnt1);
    state ^= *toggle;

    return state;
}
*/





